import { Vue, Component, Mixins } from 'vue-property-decorator'
import { fetchStudentExportData, StudentExportItem, InstanceExportData, fetchInstanceExportData } from '@/api'
import { getters } from '@/store/getters'
import { mutations as mut } from '@/store/mutations'

export enum ExportType {
  JSON = 'JSON',
  CSV = 'CSV'
}

@Component
export class parseCSVMapMixin extends Vue {
  parseCSVMap (csvMap: string) {
    console.log('parsing')
    let lines = csvMap.split('\n')
    lines.shift() // drop the first line since it contains only headings
    // TODO remove any type
    return lines.reduce((acc: any, curr) => {
      if (curr) {
        let [key, matrikelNo, name] = curr.split(';')
        acc[key] = { matrikelNo: matrikelNo, name }
      }
      return acc
    }, {})
  }
}

@Component
export class exportMixin extends Mixins(Vue, parseCSVMapMixin) {
  exportDialog = true
  mapFile: File | null = null
  setPasswords = false
  exportType = ExportType.CSV

  get mapFileLoaded () {
    return Object.keys(getters.state.studentMap).length > 0
  }

  get availableExportTypes (): ExportType[] {
    return Object.values(ExportType)
  }

  async getExportFile (type: string) {
    let studentData
    if (type === 'data') {
      studentData = await fetchStudentExportData({ setPasswords: this.setPasswords })
    } else if (type === 'instance') {
      studentData = await fetchInstanceExportData()
    } else {
      throw new Error('Unsupported export type')
    }

    if (this.mapFile || this.mapFileLoaded) {
      this.getMappedExportFile(studentData)
    } else {
      this.optionalConvertAndCreatePopup(studentData)
    }
  }

  jsonToCSV (studentExport: StudentExportItem[], delimeter = ';') {
    let headerLine = Object.keys(studentExport[0]).reduce((acc: string, curr) => {
      if (curr === 'Scores') {
        return acc
      }
      return acc ? `${acc};${curr}` : `${curr}`
    }, '')
    headerLine += Object.values(studentExport[0].Scores)
      .reduce((acc: string, curr) => {
        return `${acc};${curr.type}`
      }, '')

    const lines = studentExport.map(student => {
      const normalFields = Object.values(student).reduce((acc: string, curr): string => {
        // skip  the Scores field
        if (typeof curr === 'object') {
          return acc
        }
        return acc ? `${acc};${curr}` : `${curr}`
      }, '')

      const scoreFields = Object.values(student.Scores).reduce((acc: string, curr) => {
        return `${acc};${curr.score}`
      }, '')
      return normalFields + scoreFields
    })

    return headerLine + lines.reduce((acc, curr) => {
      return `${acc}\n${curr}`
    }, '') + '\n' // add trailing newline
  }

  optionalConvertAndCreatePopup (studentData: StudentExportItem[] | InstanceExportData) {
    const convertedData = this.exportType === ExportType.CSV
      ? this.jsonToCSV(studentData as StudentExportItem[]) : studentData
      // we have a cast here because only student export may be converted to csv

    this.createDownloadPopup(convertedData, this.exportType)
  }

  async getMappedExportFile (studentData: StudentExportItem[] | InstanceExportData) {
    if (!this.mapFile && !this.mapFileLoaded) {
      throw new Error('Either mapFile must be selected or already loaded ' +
                      'to call getMappedExportFile')
    }
    if (this.mapFile) {
      await this.readMapFileAndCommit()
    }
    this.applyMapping(studentData)
    this.optionalConvertAndCreatePopup(studentData)
  }

  readMapFileAndCommit () {
    const fileReader = new FileReader()
    return new Promise((resolve, reject) => {
      fileReader.onload = event => {
        // @ts-ignore typings of EventTarget seem to be wrong
        const studentMap = this.parseCSVMap(event.target.result)
        mut.SET_STUDENT_MAP(studentMap)
        resolve()
      }
      fileReader.onerror = () => {
        fileReader.abort()
        reject(new Error('Problem parsing input file.'))
      }

      if (!this.mapFile) {
        reject(new Error('Can only call' +
          ' readMapFileAndCommit when mapFile is not undefined'))
      } else {
        fileReader.readAsText(this.mapFile)
      }
    })
  }

  hide () {
    this.$emit('hide')
  }

  showDialog () {
    this.exportDialog = true
  }

  applyMapping (exportData: StudentExportItem[] | InstanceExportData): void { throw new Error('Not implemented.') }
  createDownloadPopup (content: string | StudentExportItem[] | InstanceExportData, fileType: ExportType): void { throw new Error('Not implemented.') }
}
